import datetime
import random

import panikweb.utils
import panikweb.views
from django.conf import settings
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _
from django.views.generic.base import TemplateView
from emissions.models import Diffusion, Emission, Episode, Focus, NewsItem, Schedule
from emissions.utils import period_program


class Home(panikweb.views.Home):
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        # order by creation timestamp
        context['shows'] = (
            Emission.objects.filter(archived=False)
            .exclude(categories__slug='collection')
            .order_by('-creation_timestamp')
        )
        # or by latest diffusion (this requires episodes)
        context['shows'] = (
            Emission.objects.filter(archived=False)
            .exclude(categories__slug='collection')
            .extra(
                select={
                    'latest_diffusion': 'emissions_diffusion.datetime',
                },
                select_params=(False, True),
                where=[
                    '''datetime = (SELECT MAX(datetime)
                                     FROM emissions_diffusion, emissions_episode
                                    WHERE episode_id = emissions_episode.id
                                      AND emission_id = emissions_emission.id)'''
                ],
                tables=['emissions_diffusion', 'emissions_episode'],
            )
            .order_by('-latest_diffusion')
            .distinct()
        )

        if context['shows'].count() > settings.HOME_EMISSIONS_COUNT:
            context['shows'] = context['shows'][:12]
            context['shows_display_more'] = True

        context['collections'] = Emission.objects.filter(archived=False).filter(
            categories__slug='collection'
        )[: settings.HOME_EMISSIONS_COUNT]
        if context['collections'].count() > settings.HOME_EMISSIONS_COUNT:
            context['collections'] = context['collections'][:12]
            context['collections_display_more'] = True

        context['newsitems'] = (
            NewsItem.objects.exclude(date__gt=datetime.date.today())
            .exclude(expiration_date__lt=datetime.date.today())
            .order_by('-date')
        )
        if context['newsitems'].count() > settings.HOME_NEWSITEMS_COUNT:
            context['newsitems'] = context['newsitems'][:12]
            context['newsitems_display_more'] = True

        context['focus'] = list(
            Focus.objects.filter(current=True).select_related(
                'emission', 'newsitem', 'soundfile', 'episode', 'newsitem__category'
            )
        )

        # create focus objects for episode scheduled for new broadcasting day.
        diffusions = (
            Diffusion.objects.filter(datetime__gt=now())
            .order_by('datetime')
            .select_related('episode', 'episode__emission')[: settings.HOME_FOCUS_COUNT]
        )
        context['focus'].extend([Focus(episode=x.episode) for x in diffusions])

        random.shuffle(context['focus'])
        context['focus'] = context['focus'][: settings.HOME_FOCUS_COUNT]

        return context


home = Home.as_view()


class Emissions(panikweb.views.Emissions):
    title = _('All Shows')

    def get_queryset(self):
        return super().get_queryset().exclude(categories__slug='collection')


emissions = Emissions.as_view()


class Collections(panikweb.views.Emissions):
    title = _('All Collections')

    def get_queryset(self):
        return super().get_queryset().filter(categories__slug='collection')


collections = Collections.as_view()


class TagItems(TemplateView):
    template_name = 'tag-items.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['tag'] = kwargs['slug']
        context['emissions'] = Emission.objects.filter(tags__slug__in=[kwargs['slug']])
        context['episodes'] = Episode.objects.filter(tags__slug__in=[kwargs['slug']])
        return context


tag_items = TagItems.as_view()


class ComingSoon(TemplateView):
    template_name = 'comingsoon.html'


coming_soon = ComingSoon.as_view()


class Calendar(TemplateView):
    template_name = 'calendar.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        year = datetime.date.today().isocalendar()[0]
        week = datetime.date.today().isocalendar()[1]

        date = panikweb.utils.tofirstdayinisoweek(year, week)
        date = datetime.datetime(*date.timetuple()[:3])

        context['weekdays'] = [datetime.date(2018, 1, x) for x in range(1, 8)]

        nb_lines = 2 * 24  # the cells are half hours
        times = ['%02d:%02d' % (x / 2, x % 2 * 30) for x in range(nb_lines)]
        times = (
            times[2 * Schedule.DAY_HOUR_START + (1 if Schedule.DAY_MINUTE_START else 0) :]
            + times[: 2 * Schedule.DAY_HOUR_START + (1 if Schedule.DAY_MINUTE_START else 0)]
        )

        context['times'] = times

        context['weeks'] = weeks = []
        for week in range(4):
            start_date = date + datetime.timedelta(days=7 * week)
            program = period_program(
                start_date,
                start_date + datetime.timedelta(days=7),
                prefetch_categories=False,
                include_nonstop=False,
            )
            grid = []
            weeks.append(
                {'start_date': start_date, 'end_date': start_date + datetime.timedelta(days=6), 'grid': grid}
            )
            start_time = datetime.time(Schedule.DAY_HOUR_START, Schedule.DAY_MINUTE_START)
            base_start_time = datetime.datetime.combine(start_date, start_time)
            for line_no in range(48):
                grid_row = []
                grid.append(grid_row)
                for column_no in range(7):
                    cell_start_time = base_start_time + datetime.timedelta(
                        days=column_no, minutes=line_no * 30
                    )
                    cell_end_time = cell_start_time + datetime.timedelta(minutes=30)
                    shows = [
                        x for x in program if x.datetime < cell_end_time and x.end_datetime > cell_start_time
                    ]
                    if shows:
                        grid_cell = getattr(shows[0], 'grid_cell', None)
                        if grid_cell:
                            grid_cell['rowspan'] += 1
                        else:
                            shows[0].grid_cell = {'show': shows[0], 'rowspan': 1}
                            grid_row.append(shows[0].grid_cell)
                    else:
                        grid_row.append({'empty': True})

        return context


calendar = Calendar.as_view()
